@@ -19,24 +19,12 @@ import geopandas as gpd
 import pandas as pd
 from sqlalchemy import create_engine
 from functools import lru_cache, partial
-import s2sphere
+from cached_property import cached_property_with_ttl
 
-from mundipy.cache import spatial_cache_footprint, pyproj_transform
+from mundipy.cache import (spatial_cache_footprint, pyproj_transform,
+	union_spatial_cache)
 from mundipy.geometry import from_dataframe, from_row_series
 
-r = s2sphere.RegionCoverer()
-
-def tile_bbox(polygon):
-	bbox = polygon.bounds
-
-	p1 = s2sphere.LatLng.from_degrees(bbox[3], bbox[0])
-	p2 = s2sphere.LatLng.from_degrees(bbox[1], bbox[2])
-
-	rect = s2sphere.LatLngRect.from_point_pair(p1, p2)
-
-	# 14 is ~0.3km2 and 600m edge length
-	return r.get_simple_covering(rect, p1.to_point(), 14)
-
 class Dataset:
 	"""
 	A Dataset represents a source of vector features.
@@ -57,8 +45,6 @@ class Dataset:
 		self._db_url = None
 		self._db_table = None
 
-		self._conn = None
-
 		if isinstance(data, dict):
 			self._db_url = data['url']
 			self._db_table = data['table']
@@ -67,66 +53,53 @@ class Dataset:
 		else:
 			raise TypeError('data for Dataset() is neither filename nor dict with PostgreSQL details')
 
-	@spatial_cache_footprint
-	def _load(self, bbox, pcs='EPSG:4326'):
+	@cached_property_with_ttl(ttl=10)
+	def _conn(self):
+		"""SQLAlchemy connection to the database, if _db_url is set"""
+
+		if self._db_url is None:
+			raise TypeError('Dataset._conn called when no _db_url was set')
+
+		return create_engine(self._db_url)
+
+	@union_spatial_cache
+	def _load(self, geom, pcs='EPSG:4326'):
 		"""
 		Load part or the entire Dataset as a dataframe.
 
-		Takes bbox as a 4-tuple of WGS84 coordinates (lon, lat). bbox can be
-		None to load the entire dataset.
+		Takes geom as a shapely.geometry, or None to load the
+		entire dataset.
 
 		Returns the dataset in PCS coordinates.
 		"""
-		if self._db_url is not None:
-			# load from PostGIS
-			self._conn = create_engine(self._db_url)
 
-			# no bbox
-			if bbox is None:
+		# convert from WGS84 to local PCS
+		transformer = partial(transform, pyproj_transform('EPSG:4326', pcs))
+
+		if self._db_url is not None:
+			# no geom
+			if geom is None:
 				# build the query
 				query = "SELECT * FROM %s" % self._db_table
 
 				gdf = gpd.GeoDataFrame.from_postgis(query, self._conn, geom_col='geometry', crs='EPSG:4326')
-				self._conn = None
-				return (gdf.to_crs(pcs), None)
+				gdf.set_geometry(gdf.geometry.apply(transformer), inplace=True, crs=pcs)
+				return gdf
 
-			cell_ids = list(tile_bbox(bbox))
+			# load entire geometry
+			query = "SELECT * FROM %s WHERE geometry && ST_GeomFromEWKT('SRID=4326;%s')" % (self._db_table, geom.wkt)
 
-			tiles = [self._load_tile(cellid, pcs) for cellid in cell_ids]
-			together = pd.concat(tiles)
-			together_geo = gpd.GeoDataFrame(data=together, geometry='geometry', crs=pcs)
+			gdf = gpd.GeoDataFrame.from_postgis(query, self._conn, geom_col='geometry', crs='EPSG:4326')
+			gdf.set_geometry(gdf.geometry.apply(transformer), inplace=True, crs=pcs)
+			return gdf
 
-			self._conn = None
-
-			# build footprint from cell_ids
-			# needs 5 points to have a box polygon
-			polygons = [[s2sphere.LatLng.from_point(s2sphere.Cell(cellid).get_vertex(v)) for v in [0, 1, 2, 3, 0]] for cellid in cell_ids]
-			polygons = [Polygon([Point(v.lng().degrees, v.lat().degrees) for v in polygon]) for polygon in polygons]
-
-			return (together_geo, MultiPolygon(polygons).buffer(0))
-
-		if bbox is None:
-			return (gpd.read_file(self.filename).to_crs(pcs), None)
+		if geom is None:
+			gdf = gpd.read_file(self.filename)
 		else:
-			return (gpd.read_file(self.filename, bbox=bbox).to_crs(pcs), None)
+			gdf = gpd.read_file(self.filename, bbox=geom)
 
-	@lru_cache(maxsize=512)
-	def _load_tile(self, cellid, pcs):
-		vertices = [s2sphere.LatLng.from_point(s2sphere.Cell(cellid).get_vertex(v)) for v in range(4)]
-
-		wkt = "POLYGON ((%f %f, %f %f, %f %f, %f %f, %f %f))" % (
-			vertices[0].lng().degrees, vertices[0].lat().degrees,
-			vertices[1].lng().degrees, vertices[1].lat().degrees,
-			vertices[2].lng().degrees, vertices[2].lat().degrees,
-			vertices[3].lng().degrees, vertices[3].lat().degrees,
-			vertices[0].lng().degrees, vertices[0].lat().degrees,
-		)
-		# shapely.wkt.loads(wkt)
-
-		query = "SELECT * FROM %s WHERE geometry && ST_GeomFromEWKT('SRID=4326;%s')" % (self._db_table, wkt)
-
-		gdf = gpd.GeoDataFrame.from_postgis(query, self._conn, geom_col='geometry', crs='EPSG:4326')
-		return gdf.to_crs(pcs)
+		gdf.set_geometry(gdf.geometry.apply(transformer), inplace=True, crs=pcs)
+		return gdf
 
 	@property
 	def dataframe(self):
@@ -135,7 +108,7 @@ class Dataset:
 
 	@lru_cache(maxsize=8)
 	def local_dataframe(self, pcs):
-		return self.dataframe.to_crs(pcs)
+		return self._load(None, pcs=pcs)
 
 	@lru_cache(maxsize=8)
 	def geometry_collection(self, pcs):
@@ -209,7 +182,7 @@ class LayerView:
 		to_wgs = pyproj_transform(self.pcs, 'EPSG:4326')
 
 		# increasing look outside of this bbox for the nearest item
-		buffer_distances = [1e0, 1e1, 1e2, 1e3, 1e4, 1e8]
+		buffer_distances = [1e3, 1e4, 1e5, 1e6, 1e7, 1e8]
 		for buffer_size in buffer_distances:
 			# buffer geom.bbox
 			bbox = transform(to_wgs, geom.buffer(buffer_size)).bounds
@@ -220,4 +193,11 @@ class LayerView:
 
 				return from_row_series(res[1])
 
+		# fuck it, check the whole dataframe
+		gdf = self.layer.local_dataframe(self.pcs)
+		if len(gdf) > 0:
+			res = min(gdf.iterrows(), key=lambda row: geom.distance(row[1].geometry))
+
+			return from_row_series(res[1])
+
 		return None
